struct result PSO (struct param param, struct problem pb) 
{  
  int d; // Current dimension
  double diam; // Swarm diameter, for tests;
  double error;   // Current error
  double errorPrev; // Previous error
  int g;   // best informant       
  int iter; 		// Iteration number (time step)
  int m;
  int nNeigh; // Number of neighbours (informants) 
  int noStop;
  int PSO_type;
  struct result R;
  int randCase;
  int rb; // Random bit
  int  s,s1; 

  double xMax,xMin;
  double zz;

  // -----------------------------------------------------
  // INITIALISATION
 // R.SW.S = param.S; // Size of the current swarm
 PSO_type=param.BW[1];
  randCase=param.BW[2]; // Either a "normal" RNG, for tests, or a list
  nNeigh=param.topology; // Ring, one or two neighbours

// Position and velocity
R.SW=init(pb.SS, param.S,param.BW[0], randCase);

  // Take quantisation into account
  if(pb.SS.quantisation==1)
  {
    for (s = 0; s < R.SW.S; s++)
      R.SW.X[s] = quantis (R.SW.X[s], pb.SS);
  }

  // First evaluations
  errMax=0;
  errMin=infinity;
  for (s = 0; s < R.SW.S; s++) 
  {	
    R.SW.X[s].f =
      perf (R.SW.X[s], pb.function,pb.SS,pb.objective);
    R.SW.P[s] = R.SW.X[s];	// Best position = current one
  }

  // If the number max of evaluations is smaller than 
  // the swarm size, just keep evalMax particles, and finish
  if (R.SW.S>pb.evalMax) R.SW.S=pb.evalMax;	
  R.nEval = R.SW.S;

  // Find the best
  R.SW.best = 0;
  errorPrev =R.SW.P[R.SW.best].f; // "distance" to the wanted f value (objective)

  for (s = 1; s < R.SW.S; s++)     
  {
    zz=R.SW.P[s].f;
    if (zz < errorPrev)
    {
      R.SW.best = s;
      errorPrev=zz;
    } 
  }

  // Display the best
  printf( "\nBest value after init. %1.20e ", errorPrev ); printf(" ");
 // 	printf( "\n Position :\n" );
//  	for ( d = 0; d < pb.SS.D; d++ ) printf( " %f", R.SW.P[R.SW.best].x[d] );

  // ---------------------------------------------- ITERATIONS	
  noStop = 0;	
  error=errorPrev;	
  iter=0;

  // Topology
  // Mono-directional ring
  for (s = 0; s < R.SW.S-1; s++) R.SW.neigh[s][0]=s+1;
  R.SW.neigh[R.SW.S-1][0]=0;

  // Bidirectional ring
  if(nNeigh==2) // Add a second neighbour
  {
    for (s = 1; s < R.SW.S; s++) R.SW.neigh[s][1]=s-1;
    R.SW.neigh[0][1]=R.SW.S-1; 
  }

  while (noStop == 0) 
  {	
    iter=iter+1;

    // Loop on particles
    for (s = 0; s < R.SW.S; s++)	// For each particle ...
    {				
     
     switch(PSO_type)
     {
      default:
      case 0: // Based on SPSO 2007
      g=s; // each particle informs itself
      for(m=0;m<nNeigh;m++) // My neighbours may be nice advisers too
      {
        s1=R.SW.neigh[s][m];
        if (R.SW.P[s1].f < R.SW.P[g].f) g = s1;
      }
      
         //... compute the new velocity
        for (d = 0; d < pb.SS.D; d++)
      {
        R.SW.V[s].v[d]=param.w *R.SW.V[s].v[d]
          + alea(0,param.c,randCase)*(R.SW.P[s].x[d] - R.SW.X[s].x[d])
          + alea(0,param.c,randCase)*(R.SW.P[g].x[d] - R.SW.X[s].x[d]);
      }
      break;
      
      case 1: // Based on PSODR (Discrete Recombination, gbest variant)

       for (d = 0; d < pb.SS.D; d++)
      {
        rb=alea_integer(0,1,randCase);       // 0 or 1 at random
       // if(alea(0,1,randCase)<0.5) rb=0; else rb=1;
        R.SW.V[s].v[d]=param.w *R.SW.V[s].v[d]
          + (R.SW.P[R.SW.neigh[s][rb]].x[d] - R.SW.X[s].x[d])
          + (R.SW.P[R.SW.best].x[d] - R.SW.X[s].x[d]);
      }
      
      break;
}
      // ... and move
      for (d = 0; d < pb.SS.D; d++) 
        R.SW.X[s].x[d] = R.SW.X[s].x[d] + R.SW.V[s].v[d];			

      // If too many evaluation, finish
      if (R.nEval >= pb.evalMax)  goto end;

      // --------------------------

      // Confinement. // Keep the particle inside the search space
                      // and modify the velocity to go back inside
                      // WARNING. In this simplified version, and in case
                      // of No confinement, the position is evaluated
                      // even if the particle is outside the search space
                      // For some functions, it may cause some problems
                      // (think at sqrt(x) for x<0 ...)
      if(param.confin!=0) 
      {		
        for (d = 0; d < pb.SS.D; d++)
        {	
         xMin=pb.SS.min[d]; xMax=pb.SS.max[d];
          if (R.SW.X[s].x[d] < xMin)
          {	
            R.SW.X[s].x[d] = xMin;
            R.SW.V[s].v[d] = -0.5*R.SW.V[s].v[d];
          }
          else
          {
            if (R.SW.X[s].x[d] > xMax)
            {			
              R.SW.X[s].x[d] = xMax;
              R.SW.V[s].v[d] = -0.5*R.SW.V[s].v[d];
            }
          }				
        }
      }	

  // Take into account that some variables are discrete
      if(pb.SS.quantisation==1)
          R.SW.X[s] = quantis (R.SW.X[s], pb.SS);

      // Evaluation
      R.SW.X[s].f =perf(R.SW.X[s],pb.function, pb.SS,pb.objective);
      R.nEval = R.nEval + 1;		
	 if ( R.nEval > pb.evalMax) goto end;
	 
      // ... update the best previous position		
      if (R.SW.X[s].f < R.SW.P[s].f)	// Improvement
      {																				
        R.SW.P[s] = R.SW.X[s];

        // ... update the best of the bests
        if (R.SW.P[s].f < R.SW.P[R.SW.best].f)
        {		
          R.SW.best = s;			
        }			
      }		

error = R.SW.P[R.SW.best].f;

 if (error < pb.epsilon) goto end;
 
      if(param.trace==1)
      {
        // Keep trace of every position, for further tests
//if(s==0)
{
        fprintf(f_trace,"%i %i ",iter,s);
        for (d = 0; d < pb.SS.D; d++) fprintf(f_trace,"%f ",R.SW.X[s].x[d]);
        for (d = 0; d < pb.SS.D; d++) fprintf(f_trace,"%f ",R.SW.P[g].x[d]);
        fprintf(f_trace,"\n");
 }
      }	
    }			// End of "for (s=0 ...  "	


    if(param.trace==2) // Swarm diameter, for tests
    {
      diam=0;
      for (s = 0; s < R.SW.S; s++)
        for (s1 = 0; s1 < R.SW.S; s1++)
      { 
        zz=distanceL(R.SW.X[s],R.SW.X[s1],2);
        if(zz>diam) diam=zz;
      }
      fprintf(f_trace,"%f\n",diam);
    }

    // Check if finished
    error = R.SW.P[R.SW.best].f;
    errorPrev = error;
    
end:

      if (error > pb.epsilon && R.nEval < pb.evalMax)
    {
      noStop = 0;	// Won't stop
    }
      else
    {
      noStop = 1;	// Will stop
    }


  } // End of "while nostop ...

   printf( "\n and the winner is ... %i, error=%e", R.SW.best, error );			
  R.error = error;
  return R;  
}
// ===========================================================
struct position quantis (struct position x, struct SS SS ) 
{     
  /*
   Quantisation of a position
   Only values like x+k*q (k integer) are admissible 
     */ 
  int d;
  double qd;
  struct position quantx;

  quantx = x;     
  for (d = 0; d < x.size; d++)
  {
    qd = SS.q.q[d];	

    if (qd > zero)	// Note that qd can't be < 0
    {           				
      quantx.x[d] = qd * floor (0.5 + x.x[d] / qd);	    
    }	
  }
  return quantx;    
}
//==============================================================================
struct swarm init(struct SS SS, int S, int option, int randCase)
{
  // Position and velocity
  int d;
  double dx;
  int index[S_max];
  int s;
  struct swarm SW;
  double xMax, xMin;
  double xTemp[S_max];

  for (s = 0; s < S; s++)   
  {
    SW.X[s].size = SS.D;
    SW.V[s].size = SS.D;
  }

  switch(option)
  {
    default:
    case 0:
      for (s = 0; s < S; s++)   
    {	
      for (d = 0; d < SS.D; d++)  
      {  
        xMin=SS.min[d]; xMax=SS.max[d];
        SW.X[s].x[d] = alea (xMin,xMax,randCase);	
        SW.V[s].v[d] = alea( xMin-SW.X[s].x[d], xMax-SW.X[s].x[d],randCase ); 
        // So that  xMin<= x+v <= xMax
      }
    }

      break; 

      case 1: // Less random

        for (d = 0; d < SS.D; d++)  // Diagonal  + random permutation
    {	
      xMin=SS.min[d]; xMax=SS.max[d]; dx=(xMax-xMin)/(S+1);
      for (s = 0; s < S; s++) xTemp[s] = xMin+s*dx;

      aleaIndex(index, S, randCase);
      for (s = 0; s < S; s++) SW.X[s].x[d] = xTemp[index[s]];
    }

      for (s = 0; s < S; s++)  
    {	
      for (d = 0; d < SS.D; d++)  
      {  
        xMin=SS.min[d]; xMax=SS.max[d]; dx=(xMax-xMin)/(S+1);
        SW.V[s].v[d] = alea( xMin-SW.X[s].x[d], xMax-SW.X[s].x[d],randCase ); 
        // So that  xMin<= x+v <= xMax
      }
    }
      break;
  }
  SW.S=S;

 
  // Display the swarm
  printf("\nPSO 319 nCycle %i",nCycle);
/*
  printf("\n Swarm");
        for (s = 0; s < S; s++)   
    {	
     printf("\n");
      for (d = 0; d < SS.D; d++)  
      {  
      printf(" %f/%f",SW.X[s].x[d], SW.V[s].v[d]);
      }   
    }
*/
  return SW;
}

