/*
nanoPSO for real world problems

 Contact for remarks, suggestions etc.:
MauriceClerc@writeme.com

Versions:
2012-01-05 First version

	For more details, see ReadMe.txt		
*/

#include "main.h"

// =================================================
int main () 
{ 
	struct position bestBest; // Best position over all runs
	int d;			// Current dimension
	double D;
	double error;			// Current error
	double errorMean;		// Average error
	double errorMin;		// Best result over all runs
	double errorMeanBest[R_max]; 
	double evalMean;		// Mean number of evaluations
	int functionCode;
	int func[funcMax]; // List of functions (codes) to optimise
	int indFunc;

	int nbFunc;
	int nFailure;		// Number of unsuccessful runs
	struct param param;
	struct problem pb; 
  int randCase;
	int run, runMax; 
	struct result result; 
	
	int scanNb;
	double success[funcMax];
	double successRate;
	int t;
	double variance;
	float z;
	double zz;

	pi = acos ((long double) -1); 
	errMax=0;
	nbRand=0;
	seed_rand_kiss(1294404794); // Initialise the RNG KISS for reproducible results
	
	// Files
	f_run = fopen ("f_run.txt", "w");  
	f_synth = fopen ("f_synth.txt", "w");
	f_trace = fopen ("f_trace.txt", "w"); // For information


	//------------------------------------------------ PARAMETERS
			// Bells and Whistles
				// Not really part of the standard
				// May improve the performance (not always)
				// Sometimes not well mathematically founded (rules of thumbs)
				// * => suggested value
				
				// Note: here "random" means "by using either the RNG or the list
				//        as if they were producing an uniform distribution"
				
	param.BW[0]=0; 	// Initialisation option
	                // 0 => Random 
	                // 1 => quasi-random
									
	param.BW[1]=0; 	// 0 => based on SPSO 2007
	                // 1 => based on PSODR (Discrete Recombination, by J. Pena).
									
	param.BW[2]=4;	// Randomness options
	                // 0 => KISS, for comparison
									// 4 => Read on a list (f_rand_quasi.txt)

	f_rand= fopen ("L17b.txt","r"); //A list of real number, if BW[2]=4


	//--------

	param.S=40; //Swarm size or Mean swarm size (if BW[0]=1). Suggested: 40 
	param.confin=1; // 0 => no confinement. WARNING, infinite loop possible
	                //* 1 => confinement.
param.topology=2; // 1 => uni-directional ring
                  //* 2 => bi-directional ring
                  
    // Coefficients
	param.w =1./( 2 * log ((double) 2)); // 0.721
	param.c = 0.5 + log ((double) 2); // 1.193
											
  //-------------------------------------------------- False randomnesses
  switch (param.BW[2]) //
  {
    default: break;
    case 4: // Prepare a list of "magic" numbers, read on a file
    t=0;
    readRand:
    scanNb=fscanf(f_rand,"%f",&z);
    if(scanNb!=EOF) 
    {
    randNumber[t]=z;
    t=t+1;
    goto readRand;
    } 
    nCycleMax=t; 
    printf("\n%i false random numbers read on a file",nCycleMax);
    break; 
  }
	// ----------------------------------------------- PROBLEM
	param.trace=0; // If >0 more information is displayed/saved (f_trace.txt)
	// Functions to optimise
	nbFunc=1; // Number of functions
	func[0]=18;
	func[1]=18; 
	func[2]=21;  
	func[3]=25;  
	func[4]=28; 
	func[5]=-1; 
	func[6]=-1;
			
	/* (see problemDef( ) for precise definitions)
	4 Tripod
	17 Lennard-Jones
	18 Gear train
	21 Compression Spring
	25 Pressure Vessel (penalty method)
	28 Frequency modulation sound parameter identification
	-1 Flat function, for test (checking biases, in particular)

*/ 

	runMax = 10; // Numbers of runs
	if (runMax > R_max) runMax = R_max;

	for(indFunc=0;indFunc<nbFunc;indFunc++) // Loop on problems
	{
		functionCode =func[indFunc];

		// Some information
		printf ("\n Function %i ", functionCode);

		// Define the problem
		pb=problemDef(functionCode);
		if(pb.SS.D>DMax) ERROR ("Can't solve it. You should increase DMax");			

		// ----------------------------------------------- RUNS	
		errorMean = 0;	    
		evalMean = 0;	    
		nFailure = 0;	
		D=pb.SS.D;
		randCase=param.BW[2];

		switch(randCase)
		{
		case 0: // KISS, for test
		 break;
		 
		default:
		case 4: // A file contains the numbers
    nCycle=0;	
    break;	
		}

		for (run = 0; run < runMax; run++)  
		{		
			result = PSO (param, pb); //**** here is the run itself
			error = result.error;
			//nCycle=run+1; if(nCycle>nCycleMax) nCycle=0;

			if (error > pb.epsilon) // If the final error is too big => failure
				nFailure = nFailure + 1;

			// Memorize the best (useful if more than one run)
			if(run==0) bestBest=result.SW.P[result.SW.best];
			else
				if(error<bestBest.f) bestBest=result.SW.P[result.SW.best];

			// Result display
			errorMean=errorMean+error;
			printf ("\nRun %i. S %i,  Eval %f. Error %e ", run+1, param.S, result.nEval, error);
			printf(" Mean %e",errorMean/(run+1));
			zz=100*(1-(double)nFailure/(run+1));
			printf("  Success  %.2f%%",zz);

			// Best position display
			//for (d=0;d<pb.SS.D;d++) printf(" %f",result.SW.P[result.SW.best].x[d]);

			// Save result
					fprintf( f_run, "\n%i %.1f %.0f %e %e ", run+1, zz, result.nEval,  error, errorMean/(run+1) );
			// Save best position
					for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_run, " %f",  result.SW.P[result.SW.best].x[d] );

			// Compute/save some statistical information
			if (run == 0)
				errorMin = error;
			else if (error < errorMin)
				errorMin = error;
				
			evalMean = evalMean + result.nEval;	
			errorMeanBest[run] = error;	
		}		// End loop on "run"

		// ---------------------END 
		
		// Display some statistical information
		evalMean = evalMean / (double) runMax;   
		errorMean = errorMean / (double) runMax;

		printf ("\n Eval. (mean)= %f", evalMean);	
		printf ("\n Error (mean) = %e", errorMean);

		// Variance
		variance = 0;

		for (run = 0; run < runMax; run++)
			variance = variance + pow (errorMeanBest[run] - errorMean, 2);

		variance = sqrt (variance / runMax);	    
		printf ("\n Std. dev. %e", variance); 

		// Success rate and minimum value
		printf("\n Failure(s) %i  ",nFailure);

		successRate = 100 * (1 - nFailure / (double) runMax);			
		printf ("Success rate = %.2f%%", successRate);
		success[indFunc]=successRate;

		printf ("\n Best min value = %1.20e", errorMin);
		printf ("\nPosition of the optimum: ");
		for (d=0;d<pb.SS.D;d++) printf(" %.20f",bestBest.x[d]);
		

		// Save	
		fprintf (f_synth, "%i %i %.0f %e %e %f %f", functionCode, pb.SS.D,successRate,
		    errorMean, variance, evalMean,bestBest.f);
		for (d=0;d<pb.SS.D;d++) fprintf(f_synth, " %1.20e",bestBest.x[d]);  
		fprintf(f_synth,"\n");      	   

} // End "for ind[..."

printf("\n errMax : %f",errMax);

		// Repeat informations at the end of all runs on all problems
	printf("\n---------");
	printf("\n Function(s):");
		for(indFunc=0;indFunc<nbFunc;indFunc++) // Loop on problems
		{
			functionCode =func[indFunc];
			printf(" %i",functionCode);
		}

		printf("\n BW = (%i, %i)",param.BW[0],param.BW[1]);
		printf("\n Swarm size: %i ",param.S);
		printf("\n w = %f",param.w);
		printf("\n c = %f",param.c);
		printf("\n Topology = %i",param.topology);
		printf("\n %e random numbers have been used",nbRand);
		fprintf(f_run,"\nnbRand %e",nbRand);
	return 0; // End of main program
}
// ===============================================================
#include "alea.c"
#include "perf.c"
#include "problemDef.c"
#include "PSO.c"
#include "tools.c"


