#include "stdio.h"
#include "math.h"
#include <stdlib.h>
#include <time.h>

#define	DMax 30 	// Max number of dimensions of the search space
#define fMax 6				// Max number of constraints +1
#define funcMax 13		// Max number of functions sequentially solved 
#define R_max 1000		// Max number of runs
#define	S_max 50 		// Max swarm size
#define zero  1.0e-40	// To avoid numerical instabilities on some machines
#define infinity 1.e+100
# define randNbMax 1500 // When using a list of previously generated "random" numbers
#define ERROR(a) {printf("\nERROR: %s\n", a);exit(-1);}

// For the pseudo-random number generator KISS
# define ulong unsigned long // To generate pseudo-random numbers with KISS
# define RAND_MAX_KISS ((unsigned long) 4294967295) //  Needs ISO C90
ulong	rand_kiss(); 
void	seed_rand_kiss(ulong seed); 


// Structures
struct quantum 
{
	double q[DMax];
	int size; 
};

struct SS
{ 
	int D;							// Dimension of the search space (D-rectangle)
	double max[DMax];		// Maximum value on each dimension 
	double min[DMax]; 	// Minimum value on each dimension
	struct quantum q;		// Quantisation step size. 0 => continuous problem
	int quantisation; 	// Flag. Set to 1 if quantisation needed
};

struct param 
{
	int BW[4];			// For "bells and whistles", testing options
	double c;				// Confidence coefficient
	int confin;     // For test. 0 => no confinement
	int S;					// Swarm size
	int topology;   // 1 => ring, mono-directional
	                // 2 => ring, bi-direction
	int trace   ;   // option. If >0 more information is displayed and/or save
	                // (see the file f_trace.txt)
	double w;				// Inertia weight
};

struct fitness 
{
	int size; 
	double f[fMax];
};

struct position 
{ 
	double f;				// Fitness value  + constraints <=0
	int size;				// Number of dimensions  D
	double x[DMax];		// Coordinates
};

struct vector 
{  
	int size;  
	double v[DMax]; 
};

struct problem 
{ 
	double epsilon; 					// Admissible error
	int evalMax; 							// Maximum number of fitness evaluations
	int function; 						// Function code
	double objective; 				// Objective value
	struct SS SS;							// Search space
};

struct swarm 
{ 
	int best; 					// rank of the best particle
	struct position P[S_max];	// Previous best positions found by each particle
	int S; 						// Swarm size 
	struct vector V[S_max];	// Velocities
	struct position X[S_max];	// Positions 
	int neigh[S_max][2]; // One or two neighbours, for ring topologies                    
};

struct result 
{  
	double nEval; 		// Number of evaluations  
	struct swarm SW;	// Final swarm
	double error;		// Numerical result of the run
};



// --------------- Sub-programs (lexicographic order)

// In alea.c
double alea (double a, double b, int option);
void aleaIndex(int index[], int S,int randCase);
int alea_integer (int a, int b, int randCase) ;

// In lennard_jones.c (included in perf.c)
double lennard_jones (struct position x); 

// In perf.c
struct fitness constraint(struct position x, int functCode);
double perf (struct position x, int function,struct SS SS, double objective);	

// In problemDef.c
struct problem problemDef(int functionCode);

// In PSO.c
struct swarm init(struct SS SS, int S, int option, int randCase);
struct result PSO ( struct param param, struct problem problem);
struct position quantis (struct position x, struct SS SS);

// In tools.c
double distanceL(struct position x1, struct position x2, double L);
int sign (double x);


// --------------- Global variables	
double errMax,errMin;
double nbRand; // For information. Number of random numbers used for
							// sequence of runs
int nCycleMax;		// Max length of the cycle, when reading the random numbers on a file
int nCycle; // Current rank of the "random" number on the file
long double	pi ; // Useful for some test functions
double randNumber[randNbMax]; // List, initially read on a file

// --------------- Files
FILE * f_run; // Results. One line by run
FILE * f_synth; // Best solution, one line by problem
FILE * f_rand; // List of false random numbers
FILE * f_trace; // To save some information for future examination