double perf (struct position xs, int function, struct SS SS, double objective) 
{				// Evaluate the fitness value for the particle of rank s  
  double c;
  int d;
  struct fitness ff={0};
  double f, x1, x2,x3,x4,x5,x6;
  double s11,s12,s21,s22;
  double theta;
  double z;

  switch (function)
  {
    case -1:		// Constant. For test of biases
      f = 0; 
    break;

    case 4:		// 2D Tripod function
      // Note that there is a big discontinuity right on the solution point. 
      x1 = xs.x[0] ; 
    x2 = xs.x[1];  
    s11 = (1.0 - sign (x1)) / 2; // Be careful, see the special sign function
    s12 = (1.0 + sign (x1)) / 2; // in tools.c
    s21 = (1.0 - sign (x2)) / 2;
    s22 = (1.0 + sign (x2)) / 2;

    f = s21 * (fabs (x1) +fabs(x2+50)); // Solution on (0,-50)  
    f = f + s22 * (s11 * (1 + fabs (x1 + 50) +
        fabs (x2 - 50)) + s12 * (2 +
            fabs (x1 - 50) +
            fabs (x2 - 50)));	  

    break;	
    case 17: // Lennard-Jones
      f=lennard_jones(xs);
    break;

    case 18: // Gear train
      f=pow(1./6.931 -xs.x[0]*xs.x[1]/(xs.x[2]*xs.x[3]),2);
//    	f=pow(fabs(1./7.5 -xs.x[0]*xs.x[1]/(xs.x[2]*xs.x[3])),2);

    break;

    case 21: // Coil compression spring  (penalty method)
      // Ref New Optim. Tech. in Eng. p 644

      x1=xs.x[0]; // {1,2, ... 70}
    x2= xs.x[1];//[0.6, 3]
    x3= xs.x[2];// relaxed form [0.207,0.5]  dx=0.001
    // In the original problem, it is a list of acceptable values
    // {0.207,0.225,0.244,0.263,0.283,0.307,0.331,0.362,0.394,0.4375,0.5}

    f=pi*pi*x2*x3*x3*(x1+2)*0.25;

    // Constraints
    ff=constraint(xs,function);

    if (ff.f[1]>0) {c=1+ff.f[1]; f=f*c*c*c;}
    if (ff.f[2]>0) {c=1+ff.f[2]; f=f*c*c*c;}
    if (ff.f[3]>0) {c=1+ff.f[3]; f=f*c*c*c;}
    if (ff.f[4]>0) {c=1+pow(10,10)*ff.f[4]; f=f*c*c*c;}
    break;

    case 25: // Pressure vessel (penalty method)
#include "perf_pressure_vessel.c"
      break;

    case 28:// Frequency modulation sound parameter identification
      theta=2*pi/100;
    f=0;
    x1=xs.x[0];x2=xs.x[1];x3=xs.x[2];x4=xs.x[3];x5=xs.x[4];x6=xs.x[5];
    for(d=1;d<=100;d++)
    {
      z=x1*sin(x2*d*theta+x3*sin(x4*d*theta+x5*sin(x6*d*theta)))
        -sin(5*d*theta+1.5*sin(4.8*d*theta+2*sin(4.9*d*theta)));
      f=f+z*z;
    }
    break;	
  }
  //------------------
  f=fabs(f-objective);  
  if(f<errMin) errMin=f; // For information
  if(f>errMax) {if(f<infinity) errMax=f; else errMax=infinity;} // For information

  return f;
}

//==========================================================
struct fitness constraint(struct position x, int functCode)
{
  // ff[0] is defined in perf()
  // Variables specific to Coil compressing spring
  static double	Fmax=1000.0;
  static double	Fp=300;
  double Cf;
  double K;
  double sp;
  double lf;

  static double	S=189000.0;
  static double	lmax=14.0;
  static double	spm=6.0;
  static double	sw=1.25;
  static double	G=11500000;
  struct fitness ff={0};

  switch(functCode)
  {
    case 21: // Compression Spring. Constraints
      Cf=1+0.75*x.x[2]/(x.x[1]-x.x[2]) + 0.615*x.x[2]/x.x[1];
    K=0.125*G*pow(x.x[2],4)/(x.x[0]*x.x[1]*x.x[1]*x.x[1]);
    sp=Fp/K;
    lf=Fmax/K + 1.05*(x.x[0]+2)*x.x[2];

    ff.f[1]=8*Cf*Fmax*x.x[1]/(pi*x.x[2]*x.x[2]*x.x[2]) -S;
    ff.f[2]=lf-lmax;
    ff.f[3]=sp-spm;			
    ff.f[4]=sw- (Fmax-Fp)/K;
    ff.size=5;
    break;

    case 25: // Pressure vessel. Constraints
      ff.f[1]=0.0193*x.x[2]-x.x[0];
    ff.f[2]=0.00954*x.x[2]-x.x[1];
    ff.f[3]=750*1728-pi*x.x[2]*x.x[2]*(x.x[3]+(4.0/3)*x.x[2]); 
    ff.size=4;
    break;	

  }

  return ff;
}
