
//===================================================
struct problem problemDef(int functionCode)
{
	int d;
	struct problem pb;

	int nAtoms; // For Lennard-Jones problem
	static double lennard_jones[14]=
	{-1, -3, -6, -9.103852, -12.7121, -16.505384,-19.821489,-24.113360,-28.422532,
		-32.77,-37.97,-44.33,-47.84,-52.32};


	pb.function=functionCode;	
	// Default values
	// Can be modified below for each function
	pb.epsilon = 0.00000;	// Acceptable error (default). May be modified below
	pb.objective = 0;       // Objective value (default). May be modified below
//	pb.constraintNb=0;
	pb.SS.quantisation=0;		// No quantisation needed (all variables are continuous)

	// ------------------ Search space
	switch (pb.function)
	{   
			case -1:			// Flat function. Impossible problem, for test
			              // in particular to check if there are some biases
			pb.SS.D =2;//  Dimension							

		for (d = 0; d < pb.SS.D; d++)
		{   
			pb.SS.min[d] = -1; 
			pb.SS.max[d] = 1;	
			pb.SS.q.q[d] = 0;	// Relative quantisation, in [0,1].   
		}

		pb.evalMax = 100000;// Max number of evaluations for each run
		pb.epsilon= 0; // 1e-3;	
		pb.objective = -1;

		break;
			
			case 4:		// Tripod
			pb.SS.D = 2;	// Dimension

		// Boundaries
		for (d = 0; d < pb.SS.D; d++)
		{
			pb.SS.min[d] = -100;
			pb.SS.max[d] = 100;
			pb.SS.q.q[d] = 0;	
		}

		pb.evalMax = 10000; 	
		pb.epsilon=1.e-4;
		break;
		
		case 17: // Lennard-Jones
			nAtoms=5; // in {2, ..., 15}
		pb.SS.D=3*nAtoms; 
		pb.objective=lennard_jones[nAtoms-2];
		pb.evalMax = 30000; //5000+3000*nAtoms*(nAtoms-1) ; // Empirical rule
		pb.epsilon=0.01; 


		for (d = 0; d < pb.SS.D; d++)
		{
			pb.SS.min[d] = -2; 
			pb.SS.max[d] = 2; 
			pb.SS.q.q[d] = 0;	
		}
		break;

		case 18: // Gear train
			pb.SS.D=4;
		pb.objective=2.7e-12;
		pb.epsilon=1.e-11; 
		for (d = 0; d < pb.SS.D; d++)
		{
			pb.SS.min[d] = 12;
			pb.SS.max[d] = 60;
			pb.SS.q.q[d] = 1;	
		}
		pb.evalMax =2500; // 20000;
		pb.SS.quantisation=1;
		break;

		case 21 : // Compression spring
			pb.SS.D=3;

		pb.SS.min[0] = 1; pb.SS.max[0] = 70; pb.SS.q.q[0] = 1; // N
		pb.SS.min[1] = 0.6; pb.SS.max[1] = 3; pb.SS.q.q[1] = 0; // D
		pb.SS.min[2] = 0.207; pb.SS.max[2] = 0.5; pb.SS.q.q[2] = 0.001; //0.001; // d

		pb.SS.quantisation=1;
		pb.evalMax = 20000; 
		pb.epsilon = 1.e-10;	
		pb.objective = 2.6254214578; 
		break;

// case 25:
#include "pressure_vessel.c"
		
				case 28:			// Frequency modulation sound parameter identification
			pb.SS.D =6;//  Dimension							

		for (d = 0; d < pb.SS.D; d++)
		{   
			pb.SS.min[d] = -6.4; 
			pb.SS.max[d] = 6.35;	
			pb.SS.q.q[d] = 0.0;	// Relative quantisation, in [0,1].   
		}

		pb.evalMax = 50000;// Max number of evaluations for each run
		pb.epsilon= 1.e-6; //4.5; //1.e-9;	
		pb.objective = 0;

		break;
		
}


	pb.SS.q.size = pb.SS.D;

	return pb;
}
#include "lennard_jones.c"